from __future__ import annotations

from typing import List, Optional, Union, Literal, TYPE_CHECKING
from pydantic import PrivateAttr

from ._base import BaseModel, TokenPageInfo, RootModelList

if TYPE_CHECKING:
    from ..resources import Classify as ClassifyResource


class ClassifyClassParams:
    name: str
    prompts: List[str]
    options: Optional[
        List[Union[str, Literal["visual", "conversation", "text_in_video"]]]
    ] = None
    conversation_option: Optional[Union[str, Literal["semantic", "exact_match"]]] = None


class ClassifyDetailedScore(BaseModel):
    max_score: float
    avg_score: float
    normalized_score: float


class ClassifyClip(BaseModel):
    start: float
    end: float
    score: float
    option: str
    prompt: str
    thumbnail_url: Optional[str] = None
    detailed_scores: Optional[ClassifyDetailedScore] = None


class ClassifyClass(BaseModel):
    name: str
    score: float
    duration_ratio: float
    clips: Optional[List[ClassifyClip]] = None


class ClassifyVideoData(BaseModel):
    video_id: str
    classes: List[ClassifyClass]


class ClassifyPageResult(BaseModel):
    _resource: ClassifyResource = PrivateAttr()
    data: List[ClassifyVideoData]
    page_info: TokenPageInfo

    def __init__(self, resource: ClassifyResource, **data):
        super().__init__(**data)
        self._resource = resource

    def __iter__(self):
        return self

    def __next__(self) -> RootModelList[ClassifyVideoData]:
        next_page_token = self.page_info.next_page_token
        if not next_page_token:
            raise StopIteration

        res = self._resource.by_page_token(next_page_token)
        self.page_info = res.page_info
        return res.data



from typing import List, Optional

from ._base import BaseModel, RootModelList


class GenerateOpenEndedTextResult(BaseModel):
    id: str
    data: str


class GenerateSummarizeChapterResult(BaseModel):
    chapter_number: int
    start: int
    end: int
    chapter_title: str
    chapter_summary: str


class GenerateSummarizeHighlightResult(BaseModel):
    start: int
    end: int
    highlight: str


class GenerateSummarizeResult(BaseModel):
    id: str
    summary: Optional[str] = None
    chapters: Optional[RootModelList[GenerateSummarizeChapterResult]] = None
    highlights: Optional[RootModelList[GenerateSummarizeHighlightResult]] = None


class GenerateGistResult(BaseModel):
    id: str
    title: Optional[str] = None
    topics: Optional[List[str]] = None
    hashtags: Optional[List[str]] = None



from typing import Union, List, Literal, Optional, Dict, Any

from ..resource import APIResource
from .. import models
from ..util import remove_none_values


class Classify(APIResource):
    def videos(
        self,
        video_ids: List[str],
        options: List[Union[str, Literal["visual", "conversation", "text_in_video"]]],
        classes: List[models.ClassifyClassParams],
        *,
        conversation_option: Optional[
            Union[str, Literal["semantic", "exact_match"]]
        ] = "semantic",
        page_limit: Optional[int] = None,
        include_clips: Optional[bool] = None,
        threshold: Optional[Dict[str, Any]] = None,
        show_detailed_score: Optional[bool] = None,
        **kwargs,
    ) -> models.ClassifyPageResult:
        json = {
            "video_ids": video_ids,
            "options": options,
            "classes": classes,
            "conversation_option": conversation_option,
            "page_limit": page_limit,
            "include_clips": include_clips,
            "threshold": threshold,
            "show_detailed_score": show_detailed_score,
        }
        res = self._post("classify", json=remove_none_values(json), **kwargs)
        return models.ClassifyPageResult(self, **res)

    def index(
        self,
        index_id: str,
        options: List[Union[str, Literal["visual", "conversation", "text_in_video"]]],
        classes: List[models.ClassifyClassParams],
        *,
        conversation_option: Optional[
            Union[str, Literal["semantic", "exact_match"]]
        ] = "semantic",
        page_limit: Optional[int] = None,
        include_clips: Optional[bool] = None,
        threshold: Optional[Dict[str, Any]] = None,
        show_detailed_score: Optional[bool] = None,
        **kwargs,
    ) -> models.ClassifyPageResult:
        json = {
            "index_id": index_id,
            "options": options,
            "classes": classes,
            "conversation_option": conversation_option,
            "page_limit": page_limit,
            "include_clips": include_clips,
            "threshold": threshold,
            "show_detailed_score": show_detailed_score,
        }
        res = self._post("classify/bulk", json=remove_none_values(json), **kwargs)
        return models.ClassifyPageResult(self, **res)

    def by_page_token(
        self,
        page_token: str,
        **kwargs,
    ) -> models.ClassifyPageResult:
        res = self._get(f"classify/{page_token}", **kwargs)
        return models.ClassifyPageResult(self, **res)



from ._base import Object, ObjectWithTimestamp, PageInfo, TokenPageInfo, RootModelList
from .engine import Engine
from .index import Index, IndexListWithPagination
from .task import Task, TaskStatus, TaskListWithPagination
from .video import Video, VideoValue, VideoListWithPagination
from .search import (
    SearchData,
    SearchResult,
    CombinedSearchResult,
)
from .generate import (
    GenerateOpenEndedTextResult,
    GenerateSummarizeChapterResult,
    GenerateSummarizeHighlightResult,
    GenerateSummarizeResult,
    GenerateGistResult,
)
from .classify import ClassifyClassParams, ClassifyPageResult
from .embed import (
    CreateEmbeddingsResult,
    EmbeddingsTask,
    EmbeddingsTaskStatus,
    CreateEmbeddingsTaskVideoParams,
)

__all__ = [
    "Object",
    "ObjectWithTimestamp",
    "PageInfo",
    "TokenPageInfo",
    "Engine",
    "Index",
    "IndexListWithPagination",
    "Task",
    "TaskListWithPagination",
    "TaskStatus",
    "Video",
    "VideoValue",
    "VideoListWithPagination",
    "SearchData",
    "SearchResult",
    "CombinedSearchResult",
    "ClassifyClassParams",
    "ClassifyPageResult",
    "GenerateOpenEndedTextResult",
    "GenerateSummarizeChapterResult",
    "GenerateSummarizeHighlightResult",
    "GenerateSummarizeResult",
    "GenerateGistResult",
    "CreateEmbeddingsResult",
    "EmbeddingsTask",
    "EmbeddingsTaskStatus",
    "CreateEmbeddingsTaskVideoParams",
    "RootModelList",
]



from __future__ import annotations

from typing import Optional, TYPE_CHECKING, Dict, Any, List, Union, Literal
from pydantic import PrivateAttr, Extra

from ._base import ObjectWithTimestamp, BaseModel, ModelMixin, PageInfo, RootModelList

if TYPE_CHECKING:
    from ..resources import Video as VideoResource
    from . import (
        GeneratorGistResult,
        GenerateSummarizeResult,
        GenerateOpenEndedTextResult,
    )


class VideoMetadata(BaseModel):
    filename: str
    duration: float
    fps: float
    width: int
    height: int
    size: int

    class Config:
        extra = Extra.allow  # This allows extra fields


class VideoHLS(BaseModel):
    video_url: Optional[str] = None
    thumbnail_urls: Optional[List[str]] = None
    status: Optional[str] = None
    updated_at: Optional[str] = None


class VideoValue(BaseModel):
    start: float
    end: float
    value: str


class Video(ObjectWithTimestamp):
    _resource: VideoResource = PrivateAttr()
    _index_id: str = PrivateAttr()
    metadata: VideoMetadata
    hls: Optional[VideoHLS] = None

    def __init__(self, resource: VideoResource, index_id: str, **data):
        super().__init__(**data)
        self._resource = resource
        self._index_id = index_id

    # Video related methods

    def update(
        self,
        *,
        title: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> None:
        return self._resource._client.index.video.update(
            self._index_id, self.id, title=title, metadata=metadata, **kwargs
        )

    def delete(
        self,
        **kwargs,
    ) -> None:
        return self._resource._client.index.video.delete(
            self._index_id, self.id, **kwargs
        )

    def transcription(
        self, *, start: Optional[float] = None, end: Optional[float] = None, **kwargs
    ) -> RootModelList[VideoValue]:
        return self._resource._client.index.video.transcription(
            self._index_id, self.id, start=start, end=end, **kwargs
        )

    def text_in_video(
        self, *, start: Optional[float] = None, end: Optional[float] = None, **kwargs
    ) -> RootModelList[VideoValue]:
        return self._resource._client.index.video.text_in_video(
            self._index_id, self.id, start=start, end=end, **kwargs
        )

    def logo(
        self, *, start: Optional[float] = None, end: Optional[float] = None, **kwargs
    ) -> RootModelList[VideoValue]:
        return self._resource._client.index.video.logo(
            self._index_id, self.id, start=start, end=end, **kwargs
        )

    def thumbnail(self, *, time: Optional[float] = None, **kwargs) -> str:
        return self._resource._client.index.video.thumbnail(
            self._index_id, self.id, time=time, **kwargs
        )

    # Generate relate methods

    def generate_gist(
        self, types: List[Union[str, Literal["topic", "hashtag", "title"]]], **kwargs
    ) -> GeneratorGistResult:
        return self._resource._client.generate.gist(self.id, types, **kwargs)

    def generate_summarize(
        self,
        type: Union[str, Literal["summary", "chapter", "highlight"]],
        *,
        prompt: Optional[str] = None,
        **kwargs,
    ) -> GenerateSummarizeResult:
        return self._resource._client.generate.summarize(
            self.id, type, prompt=prompt, **kwargs
        )

    def generate_text(
        self,
        prompt: Optional[str] = None,
        **kwargs,
    ) -> GenerateOpenEndedTextResult:
        return self._resource._client.generate.text(
            self.id, type, prompt=prompt, **kwargs
        )


class VideoListWithPagination(ModelMixin, BaseModel):
    _resource: VideoResource = PrivateAttr()
    _origin_params: Dict[str, Any] = PrivateAttr()
    data: RootModelList[Video] = []
    page_info: PageInfo

    def __init__(self, resource: VideoResource, origin_params: Dict[str, Any], **data):
        super().__init__(**data)
        self._resource = resource
        self._origin_params = origin_params

    def __iter__(self):
        return self

    def __next__(self) -> RootModelList[Video]:
        if self.page_info.page >= self.page_info.total_page:
            raise StopIteration
        params = self._origin_params
        params["page"] = self.page_info.page + 1
        res = self._resource.list_pagination(**params)
        self.page_info = res.page_info
        return res.data



from .engine import Engine
from .index import Index
from .task import Task
from .video import Video
from .search import Search
from .classify import Classify
from .generate import Generate
from .embed import Embed, EmbedTask

__all__ = [
    "Engine",
    "Index",
    "Task",
    "Video",
    "Search",
    "Classify",
    "Generate",
    "Embed",
    "EmbedTask",
]



from __future__ import annotations
from typing import Union, List, Literal, Optional, BinaryIO, TYPE_CHECKING

from ..models._base import RootModelList
from ..resource import APIResource
from .. import models
from ..util import remove_none_values


if TYPE_CHECKING:
    from ..client import TwelveLabs


class EmbedTask(APIResource):
    def retrieve(self, id: str, **kwargs) -> models.EmbeddingsTask:
        res = self._get(f"embed/tasks/{id}", **kwargs)
        return models.EmbeddingsTask(self, **res)

    def create(
        self,
        engine_name: str,
        *,
        video_file: Union[str, BinaryIO, None] = None,
        video_url: Optional[str] = None,
        video_start_offset_sec: Optional[float] = None,
        video_end_offset_sec: Optional[float] = None,
        video_clip_length: Optional[int] = None,
        video_embedding_scopes: Optional[List[Literal["clip", "video"]]] = None,
        **kwargs,
    ) -> models.EmbeddingsTask:
        if not video_file and not video_url:
            raise ValueError("Either video_file or video_url must be provided")
        data = {
            "engine_name": engine_name,
            "video_embedding_scope": video_embedding_scopes,
            "video_url": video_url,
            "video_start_offset_sec": video_start_offset_sec,
            "video_end_offset_sec": video_end_offset_sec,
            "video_clip_length": video_clip_length,
        }

        files = {}
        opened_files: List[BinaryIO] = []
        if video_file is not None:
            if isinstance(video_file, str):
                file = open(video_file, "rb")
                opened_files.append(file)
                files["video_file"] = file
            else:
                files["video_file"] = video_file
        else:
            # Request should be sent as multipart-form even file not exists
            files["dummy"] = ("", "")

        try:
            res = self._post(
                "embed/tasks",
                data=remove_none_values(data),
                files=files,
                **kwargs,
            )
            task_id = res["_id"]
            return self.retrieve(task_id)
        finally:
            for file in opened_files:
                file.close()

    def create_bulk(
        self,
        engine_name: str,
        videos: List[models.CreateEmbeddingsTaskVideoParams],
        **kwargs,
    ) -> RootModelList[models.EmbeddingsTask]:
        tasks = []
        for video_params in videos:
            try:
                task = self.create(
                    engine_name,
                    video_file=video_params.file,
                    video_url=video_params.url,
                    video_start_offset_sec=video_params.start_offset_sec,
                    video_end_offset_sec=video_params.end_offset_sec,
                    video_clip_length=video_params.clip_length,
                    scopes=video_params.scopes,
                    **kwargs,
                )
                tasks.append(task)
            except Exception as e:
                print(f"Error creating task with video: {e}")
                continue
        return tasks

    def status(self, task_id: str, **kwargs) -> models.EmbeddingsTaskStatus:
        res = self._get(f"embed/tasks/{task_id}/status", **kwargs)
        return models.EmbeddingsTaskStatus(**res)


class Embed(APIResource):
    task: EmbedTask

    def __init__(self, client: TwelveLabs) -> None:
        super().__init__(client)
        self.task = EmbedTask(client)

    def create(
        self,
        engine_name: str,
        text: str,
        *,
        text_truncate: Literal["none", "start", "end"],
        **kwargs,
    ) -> models.CreateEmbeddingsResult:
        data = {
            "engine_name": engine_name,
            "text": text,
            "text_truncate": text_truncate,
        }
        res = self._post("embed", data=remove_none_values(data), **kwargs)
        return models.CreateEmbeddingsResult(**res)



from __future__ import annotations

from typing import List, Union, BinaryIO, Optional, Literal, Callable, TYPE_CHECKING
from pydantic import PrivateAttr

from ._base import BaseModel, Object

if TYPE_CHECKING:
    from ..resources import EmbedTask as EmbedTaskResource


class CreateEmbeddingsTaskVideoParams:
    file: Union[str, BinaryIO, None]
    url: Optional[str]
    start_offset_sec: Optional[float]
    end_offset_sec: Optional[float]
    clip_length: Optional[int]
    scopes: Optional[List[Literal["clip", "video"]]]


class Embedding(BaseModel):
    float: List[float]


class CreateEmbeddingsResult(BaseModel):
    engine_name: str
    text_embedding: Embedding


class EmbeddingsTaskStatus(Object):
    engine_name: str
    status: str


class VideoEmbedding(BaseModel):
    start_offset_sec: float
    end_offset_sec: float
    embedding_scope: str
    embedding: Embedding


class EmbeddingsTask(Object):
    _resource: EmbedTaskResource = PrivateAttr()
    engine_name: str
    status: str
    video_embeddings: Optional[List[VideoEmbedding]] = None

    def __init__(self, resource: EmbedTaskResource, **data):
        super().__init__(**data)
        self._resource = resource

    @property
    def done(self) -> bool:
        return self.status in ("ready", "failed")

    def retrieve(self, **kwargs) -> EmbeddingsTask:
        return self._resource.retrieve(self.id, **kwargs)

    def update_status(self, **kwargs) -> str:
        res = self._resource.status(self.id, **kwargs)
        self.status = res.status
        return self.status

    def wait_for_done(
        self,
        *,
        sleep_interval: float = 5.0,
        callback: Optional[Callable[[EmbeddingsTask], None]] = None,
        **kwargs,
    ) -> str:
        if sleep_interval <= 0:
            raise ValueError("sleep_interval must be greater than 0")
        while not self.done:
            self._resource._sleep(sleep_interval)
            self.update_status(**kwargs)
            if callback is not None:
                callback(self)
        return self.status



from typing import List

from ..models import RootModelList
from ..resource import APIResource
from .. import models


class Engine(APIResource):
    def retrieve(self, id: str, **kwargs) -> models.Engine:
        res = self._get(f"engines/{id}", **kwargs)
        return models.Engine(**res)

    def list(self, **kwargs) -> RootModelList[models.Engine]:
        res = self._get("engines", **kwargs)
        return RootModelList([models.Engine(**engine) for engine in res["data"]])
